import re

from jeb.api import IScript
from jeb.api.ui import View

class RenameClasses(IScript):
    def run(self, j):
        self.dex = j.getDex()
        self.jeb = j
        ui = j.getUI()

        # rename obfuscated class names
        self.rename_classes()

        # refresh view
        ui.getView(View.Type.ASSEMBLY).refresh()
        ui.getView(View.Type.JAVA).refresh()
        ui.getView(View.Type.CLASS_HIERARCHY).refresh()

    def rename_classes(self):
        for i in range(0, self.dex.getClassCount()):
            cls = self.dex.getClass(i)
            cls_name = self.dex.getType(cls.getClasstypeIndex())
            ret = self.rename_by_super_class(cls)

            if ret:
                new_name = self.dex.getType(cls.getClasstypeIndex())
                print "rename from '%s' to '%s'" % (cls_name, new_name)


    def append_cls_name(self, cls, append_str):
        p = re.compile("^.*\/([\w$]+);$")
        # cls_name has package path
        cls_name = self.dex.getType(cls.getClasstypeIndex())
        if cls_name.find(append_str) == -1:
            s = re.search(p, cls_name)
            simple_new_name = s.group(1) + '_' + append_str 
            return self.jeb.renameClass(cls_name, simple_new_name)
        else:
            return False

    def rename_by_super_class(self, cls):
        rename_targets = {
            'Landroid/app/Service;': 'Service',
            'Landroid/app/Activity;': 'Activity',
            'Landroid/content/BroadcastReceiver;': 'Receiver',
            'Landroid/content/ContentProvider;': 'Provider',
            'Ljava/lang/Thread;': 'Thread',
            'Landroid/os/AsyncTask;': 'AsyncTask',
        }
        super_cls_type = self.dex.getType(cls.getSuperclassIndex())
        if super_cls_type in rename_targets.keys():
            val = rename_targets[super_cls_type]
            return self.append_cls_name(cls, val)
        else:
            return False



